@@ -9,6 +9,8 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <unistd.h>
+#include <string.h>
 #include <errno.h>
 #include "nfslib.h"
 
@@ -22,6 +24,12 @@ main(int argc, char **argv)
 	if (argc > 1)
 		usage (argv [0]);
 
+	if (chdir(NFS_STATEDIR)) {
+		fprintf(stderr, "%s: chdir(%s) failed: %s\n",
+			argv [0], NFS_STATEDIR, strerror(errno));
+		exit(1);
+	}
+
 	if ((error = lockdsvc()) < 0) {
 		if (errno == EINVAL)
 			/* Ignore EINVAL since kernel may start
